package envoy.client.ui.list;

import javax.swing.JComponent;

/**
 * Allows a {@link ComponentList} convert its elements into Swing components
 * that can be rendered.<br>
 * <br>
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>ComponentListCellRenderer.java</strong><br>
 * Created: <strong>25.01.2020</strong><br>
 *
 * @param <E> the type of object displayed in this list
 * @author Kai S. K. Engelbart
 * @since Envoy v0.3-alpha
 */
public interface ComponentListCellRenderer<E> {

	/**
	 * Provides a Swing component representing a list element.
	 *
	 * @param list       the list in which the component will be displayed
	 * @param value      the list element that will be converted
	 * @param isSelected {@code true} if the user has selected the list cell in
	 *                   which the list element is rendered
	 * @return the component representing the list element
	 * @since Envoy v0.3-alpha
	 */
	JComponent getListCellComponent(ComponentList<? extends E> list, E value, boolean isSelected);
}
